package com.ruoyi.oa.service.impl;

import java.util.List;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.oa.mapper.CmcAssessMapper;
import com.ruoyi.oa.domain.CmcAssess;
import com.ruoyi.oa.service.ICmcAssessService;

/**
 * 考核表Service业务层处理
 * 
 * @author cmc
 * @date 2024-01-09
 */
@Service
public class CmcAssessServiceImpl implements ICmcAssessService 
{
    @Autowired
    private CmcAssessMapper cmcAssessMapper;

    /**
     * 查询考核表
     * 
     * @param assessId 考核表主键
     * @return 考核表
     */
    @Override
    public CmcAssess selectCmcAssessByAssessId(String assessId)
    {
        return cmcAssessMapper.selectCmcAssessByAssessId(assessId);
    }

    /**
     * 查询考核表列表
     * 
     * @param cmcAssess 考核表
     * @return 考核表
     */
    @Override
    public List<CmcAssess> selectCmcAssessList(CmcAssess cmcAssess)
    {
        return cmcAssessMapper.selectCmcAssessList(cmcAssess);
    }

    /**
     * 新增考核表
     * 
     * @param cmcAssess 考核表
     * @return 结果
     */
    @Override
    public int insertCmcAssess(CmcAssess cmcAssess)
    {
        return cmcAssessMapper.insertCmcAssess(cmcAssess);
    }

    /**
     * 修改考核表
     * 
     * @param cmcAssess 考核表
     * @return 结果
     */
    @Override
    public int updateCmcAssess(CmcAssess cmcAssess)
    {
        return cmcAssessMapper.updateCmcAssess(cmcAssess);
    }

    /**
     * 批量删除考核表
     * 
     * @param assessIds 需要删除的考核表主键
     * @return 结果
     */
    @Override
    public int deleteCmcAssessByAssessIds(String[] assessIds)
    {
        return cmcAssessMapper.deleteCmcAssessByAssessIds(assessIds);
    }

    /**
     * 删除考核表信息
     * 
     * @param assessId 考核表主键
     * @return 结果
     */
    @Override
    public int deleteCmcAssessByAssessId(String assessId)
    {
        return cmcAssessMapper.deleteCmcAssessByAssessId(assessId);
    }
}
